@@ -28,7 +28,7 @@ from datetime import timedelta
 from collections import namedtuple
 
 # make tuple for line, and have multiple of them in a MsgPlot, as long as their units match and they are from the same message
-LineInfo = namedtuple('LineInfo', 'fieldInfo fieldSubindex dataArray timeArray curve ptr1')
+LineInfo = namedtuple('LineInfo', 'msgClass msgKey baseName fieldInfo fieldSubindex dataArray timeArray curve ptr1')
 
 start_time = datetime.now().timestamp()
 def elapsedSeconds(timestamp):
@@ -36,6 +36,18 @@ def elapsedSeconds(timestamp):
         return timestamp - start_time
     return timestamp
 
+# I just added an optimization that replaces sequences of flat data
+# with just two points, so we're starting to be a little smarter
+# about storing data.  maybe we want to decimate very old data?
+# that'll look wrong if user zooms in, though
+# it also breaks time slider, because there's no longer the
+# same number of points in each line for the same time period :(
+# timeslider should likely change to using actual X axis ranges,
+# and then we need to find points that falls in that time in the data?
+# if time resets (which happens often when msgserver restarts or
+# embedded processor resets depending on how time is managed,
+# then time is non-linear and not necessarily always increasing in
+# the arrays and any kind of smart search won't work well :(
 class MsgPlot(QWidget):
     class PlotError(Exception):
         pass
@@ -45,8 +57,9 @@ class MsgPlot(QWidget):
 
     Paused = QtCore.pyqtSignal(bool)
     AddLineError = QtCore.pyqtSignal(str)
-    MAX_LENGTH = 500
-    def __init__(self, msgClass, fieldName, runButton = None, clearButton = None, timeSlider = None, displayControls=True):
+    RegisterForMessage = QtCore.pyqtSignal(str)
+    MAX_LENGTH = 1000
+    def __init__(self, msgClass, msgKey, fieldName, runButton = None, clearButton = None, timeSlider = None, displayControls=True, fieldLabel=None):
         super(QWidget,self).__init__()
         
         newFieldName, fieldIndex = MsgPlot.split_fieldname(fieldName)
@@ -56,9 +69,10 @@ class MsgPlot(QWidget):
 
         layout = QVBoxLayout()
         self.setLayout(layout)
-        self.msgClass = msgClass
+        #self.msgClass = msgClass
         self.pause = 0
         self.lineCount = 0
+        self.showUnitsOnLegend = True
         self.units = fieldInfo.units
         self.lines = []
 
@@ -67,7 +81,7 @@ class MsgPlot(QWidget):
         self.plotWidget = pg.PlotWidget(labels={'left':yAxisLabel,'bottom':xAxisLabel})
         layout.addWidget(self.plotWidget)
         self.plotWidget.addLegend()
-        self.addLine(msgClass, fieldName)
+        self.addLine(msgClass, msgKey, fieldName, fieldLabel)
 
         # set up click handler to pause graph
         self.plotWidget.scene().sigMouseClicked.connect(self.mouseClicked)
@@ -134,7 +148,11 @@ class MsgPlot(QWidget):
         ev.accept()
         item = ev.source().currentItem()
         try:
-            self.addLine(type(item.msg), item.fieldName)
+            # add a line for whatever got dropped on us
+            msgClass = type(item.msg)
+            self.addLine(msgClass, item.msg_key, item.fieldName)
+            # register to receive message updates for the new line
+            self.RegisterForMessage.emit(item.msg_key)
         except MsgPlot.PlotError as e:
             self.AddLineError.emit(str(e))
     
@@ -144,7 +162,7 @@ class MsgPlot(QWidget):
             line.timeArray.clear()
             self.refreshLine(line)
 
-    def addLine(self, msgClass, fieldName):
+    def addLine(self, msgClass, msgKey, fieldName, fieldLabel = None):
         fieldName, fieldIndex = MsgPlot.split_fieldname(fieldName)
         fieldInfo = Messaging.findFieldInfo(msgClass.fields, fieldName)
         if fieldInfo == None:
@@ -160,15 +178,26 @@ class MsgPlot(QWidget):
                 if fieldInfo.count > 1:
                     name = "%s[%d]" % (name, fieldIndex)
                 raise MsgPlot.PlotError("Line %s already on plot" % name)
-        if msgClass != self.msgClass:
-            raise MsgPlot.NewPlotError("Message %s != %s, cannot add to same plot" % (msgClass.__name__, self.msgClass.__name__))
-        if fieldInfo.units != self.units:
-            raise MsgPlot.NewPlotError("Units %s != %s, not adding to plot" % (fieldInfo.units, self.units))
-        
+        #TODO why not plot things from different messages?!?
+        #if msgClass != self.msgClass:
+        #    raise MsgPlot.NewPlotError("Message %s != %s, cannot add to same plot" % (msgClass.__name__, self.msgClass.__name__))
+        if self.showUnitsOnLegend:
+            if fieldInfo.units != self.units:
+                self.showUnitsOnLegend = True
+                self.units = None
+                #TODO how to edit existing left axis label?!?
+                #print(str(self.plotWidget)) # how to print all attributes of an object?
+                # pyqtgraph.widgets.PlotWidget.PlotWidget just shows up as <pyqtgraph.widgets.PlotWidget.PlotWidget object at 0x7f64c4644168>
+                #self.plotWidget.labels['left'] = 'none?!?'
+                for line in self.lines:
+                    pass
+                    #TODO how to edit existing legend
+                    #line.curve.setName(line.baseName)
+
         if fieldIndex != None:
-            self._addLine(msgClass, fieldInfo, fieldIndex)
+            self._addLine(msgClass, msgKey, fieldInfo, fieldIndex, fieldLabel, self.showUnitsOnLegend)
         elif fieldInfo.count == 1:
-            self._addLine(msgClass, fieldInfo, 0)
+            self._addLine(msgClass, msgKey, fieldInfo, 0, fieldLabel, self.showUnitsOnLegend)
         else:
             dups = []
             for fieldIndex in range(0, fieldInfo.count):
@@ -179,7 +208,7 @@ class MsgPlot(QWidget):
                         duplicate = True
                         break
                 if not duplicate:
-                    self._addLine(msgClass, fieldInfo, fieldIndex)
+                    self._addLine(msgClass, msgKey, fieldInfo, fieldIndex, fieldLabel, self.showUnitsOnLegend)
             if len(dups) > 0:
                 if len(dups) == 1:
                     s = ' '+str(dups[0])
@@ -191,8 +220,14 @@ class MsgPlot(QWidget):
                         s += '%s,' % d
                 raise MsgPlot.PlotError("Line%s already on plot" % s)
 
-    def _addLine(self, msgClass, fieldInfo, fieldIndex):
+    def _addLine(self, msgClass, msgKey, fieldInfo, fieldIndex, fieldLabel = None, showUnits=False):
         lineName = fieldInfo.name
+        baseName = lineName
+        if showUnits and fieldInfo.units != '' and fieldInfo.units != 'UNKNOWN':
+            lineName = lineName + " (%s)" % fieldInfo.units
+        if fieldLabel != None:
+            lineName = fieldLabel
+            baseName = lineName
         try:
             if fieldInfo.count != 1:
                 lineName += "["+str(fieldIndex)+"]"
@@ -202,8 +237,14 @@ class MsgPlot(QWidget):
         timeArray = []
         ptr1 = 0
         self.useHeaderTime = 0
-        curve = self.plotWidget.plot(timeArray, dataArray, name=lineName, pen=(len(self.lines)))
-        lineInfo = LineInfo(fieldInfo, fieldIndex, dataArray, timeArray, curve, ptr1)
+        # This is ugly, but try to make plotWidget.plot(pen) parameters that result in colors easy to distinguish.
+        line_number = len(self.lines)
+        line_count_estimate = 6
+        while line_count_estimate < len(self.lines)+1:
+            line_count_estimate = line_count_estimate + 6
+            line_number = (line_number - 6)*2+1
+        curve = self.plotWidget.plot(timeArray, dataArray, name=lineName, pen=(line_number,line_count_estimate))
+        lineInfo = LineInfo(msgClass, msgKey, baseName, fieldInfo, fieldIndex, dataArray, timeArray, curve, ptr1)
         self.lines.append(lineInfo)
         
     def pauseOrRun(self):
@@ -220,6 +261,8 @@ class MsgPlot(QWidget):
         # TODO what to do for things that can't be numerically expressed?  just ascii strings, i guess?
         for line in self.lines:
             try:
+                if type(msg) != line.msgClass:
+                    continue
                 newDataPoint = Messaging.getFloat(msg, line.fieldInfo, line.fieldSubindex)
             except ValueError:
                 print("ERROR! Plot of %s.%s cannot accept value %s" % (
@@ -240,16 +283,23 @@ class MsgPlot(QWidget):
                 # if header has no time, fallback to PC time.
                 newTime = elapsedSeconds(datetime.now().timestamp())
             
-            # add data in the array until MAX_LENGTH is reached, then drop data off start of array
-            # such that plot appears to scroll.  The array size is limited to MAX_LENGTH.
-            if len(line.dataArray) >= MsgPlot.MAX_LENGTH:
-                line.dataArray[:-1] = line.dataArray[1:]  # shift data in the array one sample left
-                line.dataArray[-1] = newDataPoint
-                line.timeArray[:-1] = line.timeArray[1:]  # shift data in the array one sample left
+            # if the last two data points have the same value as us, then instead of adding a point,
+            # we can just change the time of the previous point to our time
+            if (len(line.dataArray) > 2 and
+                line.dataArray[-2] == line.dataArray[-1] and
+                line.dataArray[-1] == newDataPoint):
                 line.timeArray[-1] = newTime
             else:
-                line.dataArray.append(newDataPoint)
-                line.timeArray.append(newTime)
+                # add data in the array until MAX_LENGTH is reached, then drop data off start of array
+                # such that plot appears to scroll.  The array size is limited to MAX_LENGTH.
+                if len(line.dataArray) >= MsgPlot.MAX_LENGTH:
+                    line.dataArray[:-1] = line.dataArray[1:]  # shift data in the array one sample left
+                    line.dataArray[-1] = newDataPoint
+                    line.timeArray[:-1] = line.timeArray[1:]  # shift data in the array one sample left
+                    line.timeArray[-1] = newTime
+                else:
+                    line.dataArray.append(newDataPoint)
+                    line.timeArray.append(newTime)
 
             if not self.pause:
                 self.refreshLine(line)
@@ -269,15 +319,19 @@ class MsgPlot(QWidget):
             self.refreshLine(line)
 
     @staticmethod
-    def plotFactory(new_plot, msgClass, fieldNames, runButton = None, clearButton = None, timeSlider = None, displayControls=True):
+    def plotFactory(new_plot_callback, msgClass, fieldNames, msgKey = None, fieldLabels = None, runButton = None, clearButton = None, timeSlider = None, displayControls=True):
         msgPlot = None
         if len(fieldNames) == 0:
             fieldNames = [fieldInfo.name for fieldInfo in msgClass.fields]
+        idx = 0
         for fieldName in fieldNames:
+            fieldLabel = None
+            if fieldLabels != None:
+                fieldLabel = fieldLabels[idx]
             # if there's a plot widget, try adding a line to it
             if msgPlot != None:
                 try:
-                    msgPlot.addLine(msgClass, fieldName)
+                    msgPlot.addLine(msgClass, msgKey, fieldName, fieldLabel)
                 except MsgPlot.NewPlotError as e:
                     # if error on adding to existing plot, then make a new plot
                     msgPlot = None
@@ -287,10 +341,12 @@ class MsgPlot(QWidget):
             # make new plot
             if msgPlot == None:
                 try:
-                    msgPlot = MsgPlot(msgClass, fieldName, runButton, clearButton, timeSlider, displayControls)
-                    new_plot(msgPlot)
+                    msgPlot = MsgPlot(msgClass, msgKey, fieldName, runButton, clearButton, timeSlider, displayControls, fieldLabel=fieldLabel)
+                    msgPlot.msgClass = msgClass
+                    new_plot_callback(msgPlot)
                 except MsgPlot.PlotError as e:
                     print(str(e))
+            idx += 1
         return msgPlot
 
 
